
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   cameras_calibration
%   by Petra Kocmanova
%   last modified: 27.6.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function cameras_calibration
%Function for mutual cameras calibration
%   Main inputs and results for calibration are in 2 global structure
%   arrays (images and cameras), k is number of calibrated cameras and i 
%   is number of images used for calibration 
%
%   Fields of structure array images [k x i] are following:
%   images().grid_p - images coordinates of nodal points on calibration plate
%   [px]
%   images().grid_x - object coordinates of nodal points on calibration plate
%   [m] 
%   images().H - matrix of homographie
%   images().R0 - initial value of rotation matrix between calibration plate
%   object coordinate system and cameras image coordinate system
%   images().t0 - initial value of translations between calibration plate
%   object coordinate system and cameras image coordinate system
%   images().R - refine value of rotation matrix between calibration plate
%   object coordinate system and cameras image coordinate system
%   images().t - refine value of translations between calibration plate
%   object coordinate system and cameras image coordinate system
%   images().Rm - value of rotation matrix between reference camera 
%   calibration coordinate system and coordinate system of calibrated camera
%   images().tm - value of translations between reference camera 
%   calibration coordinate system and coordinate system of calibrated camera
%
%   Fields of structure array cameras [k x 1] are following:
%   cameras.search_window - search window [px] for automatic corner
%   extraction
%   cameras().fc -initial value for focal length used only for manual corner
%   extraction
%   cameras().name - name of calibrated camera
%   cameras().K0 - initial value of camera matrix
%   cameras().K - refine value of camera matrix after first adjustment
%   cameras().k - vector of distortion coefficients
%   cameras().rep_error - vector of reprojection error after first adjustment
%   cameras().Rref - refine rotation matrix between coordinate system of
%   reference camera and coordinate system of calibrated camera
%   cameras().tref - refine translations between coordinate system of
%   reference camera and coordinate system of calibrated camera
%   cameras().Kref - refine camera matrix after second adjustment
%   cameras().rep_error_ext - reprojection error after second adjustment for
%   image coordinate x and y
%   cameras().sd_rep_error_ext - standard deviation of reprojection error 
%   after second adjustment 

global images
global use_im
global cameras

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%progress dialog
d1 = com.mathworks.mlwidgets.dialog.ProgressBarDialog.createProgressBar...
    ('Mutual cameras calibration', []);
d1.setValue(0.05);                        % default = 0
d1.setProgressStatusLabel('Processing...');  % default = 'Please Wait'
d1.setSpinnerVisible(true);               % default = true
d1.setCircularProgressBar(false);         % default = false  (true means an indeterminate (looping) progress bar)
d1.setCancelButtonVisible(false);         % default = true
d1.setVisible(true);                      % default = false
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[images_number, cameras_number]=size(images);

k0=zeros(1,5);
Hi=cell(cameras_number,images_number);

for k=1:cameras_number
    if isempty(use_im(k,:))==1
        
    else
        
        for i=1:images_number
            if use_im(k,i)
                [images(i,k).H]=compute_homography(images(i,k).grid_p,images(i,k).grid_x);
                Hi{k,i}=images(i,k).H;
            end
        end
        
        hom_im_number=size(Hi,2);
        V=zeros(2*hom_im_number,6);
        
        for i=1:hom_im_number
            Hii = Hi{k,i}/norm(Hi{k,i}(:,1));
            V(2*i-1,:) = vij(Hii,1,2)';
            V(2*i,:) = (vij(Hii,1,1)-vij(Hii,2,2))';
        end
        
        
        initial_calibration(V,Hi,k)
        cameras(k,1).K0(1,2)=0;%without skew
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %progress dialog
        d1.setValue(0.05+0.6*k/cameras_number);
        d1.setProgressStatusLabel(['Processing...refine intrinsic parameters ('...
            int2str(k) '/' int2str(cameras_number) ')' ] );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %Refine calibration
        refine_calibration(k0,k)
        
        
        %Point in front of the camera not behind
        for i=1:images_number
            if use_im(k,i)
                if(images(i,k).t(3) < 0)
                    images(i,k).R(:,1:2) = -images(i,k).R(:,1:2);
                    images(i,k).t = -images(i,k).t;
                end
            end
        end
        
        param0.K{k}=cameras(k,1).K;
        
        if(k==1)
            %TOF camera is reference for next cameras
            for i=1:images_number
                if use_im(k,i)
                    param0.Rext{1,i}=images(i,k).R;
                    param0.text{1,i}=images(i,k).t;
                end
            end
            cameras(k,1).Rref = eye(3);
            cameras(k,1).tref = zeros(3,1);
        else
            %Computation of rotation and translation matrix for other cameras
            %relative to the reference camera
            Rvm=zeros(3,images_number);
            tm=zeros(3,images_number);
            
            for i=1:images_number
                if use_im(k,i)
                    images(i,k).Rm = images(i,1).R*images(i,k).R';
                    Rvm(:,i) = rotationpars(images(i,k).Rm);
                    images(i,k).tm = images(i,1).t-images(i,k).Rm*images(i,k).t;
                    tm(:,i)=images(i,k).tm;
                end
            end
            
            cameras(k,1).Rref = rotationmat(mean(Rvm,2));
            cameras(k,1).tref = mean(tm,2);
            
            param0.R{k}=cameras(k,1).Rref;
            param0.t{k}=cameras(k,1).tref;
            
            %In case that results for camera 1 is not available
            %nutn opravit
            for i=1:images_number
                if use_im(k,i)
                else
                    images(i,k).Rext=cameras(k,1).Rref*images(i,k).R;
                    images(i,k).text=cameras(k,1).tref+cameras(k,1).Rref*images(i,k).t;
                end
            end

        end
    end
end

if cameras_number>1
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %progress dialog
    d1.setValue(0.9);
    d1.setProgressStatusLabel('Processing...refine extrinsic parameters');
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    refine_extrinsic_calibration(param0)
end

d1.setVisible(false);

end



function v=vij(H,i,j)
  v = [H(1,i)*H(1,j), H(1,i)*H(2,j)+H(2,i)*H(1,j), H(2,i)*H(2,j), ...
    H(3,i)*H(1,j)+H(1,i)*H(3,j), H(3,i)*H(2,j)+H(2,i)*H(3,j), H(3,i)*H(3,j)]';
end

